import sys
from distutils.version import LooseVersion
from unittest.mock import patch

from django.test import override_settings
from django.urls import clear_url_caches

import cms
from cms.api import add_plugin, create_page
from cms.appresolver import clear_app_resolvers
from cms.test_utils.testcases import CMSTestCase


# These means "less than or equal"
CMS_3_6 = LooseVersion(cms.__version__) < LooseVersion("4.0")


class SubmitFormViewTest(CMSTestCase):
    def setUp(self):
        self.APP_MODULE = "aldryn_forms.cms_apps.FormsApp"
        clear_app_resolvers()
        clear_url_caches()

        if self.APP_MODULE in sys.modules:
            del sys.modules[self.APP_MODULE]

        self.page = create_page(
            "tpage",
            "test_page.html",
            "en",
            published=True,
            apphook="FormsApp",
        )
        try:
            self.placeholder = self.page.placeholders.get(slot="content")
        except AttributeError:
            self.placeholder = self.page.get_placeholders("en").get(slot="content")

        self.redirect_url = "http://www.google.com"
        self.redirect_url_with_params = "http://www.google.com?aldryn_form_post_ident=aBH7hWEGAihsg9KxctpNRfvEXUoOFpJZigmZETqWWNVs4gENFsL3qva1d4Q93URg"

        plugin_data = {
            "redirect_type": "redirect_to_url",
            "url": self.redirect_url,
        }
        self.form_plugin = add_plugin(
            self.placeholder, "FormPlugin", "en", **plugin_data
        )  # noqa: E501

        add_plugin(
            self.placeholder,
            "SubmitButton",
            "en",
            target=self.form_plugin,
            label="Submit",
        )
        self.form_plugin.action_backend = "default"
        self.form_plugin.save()
        if CMS_3_6:
            self.page.publish("en")

        self.reload_urls()
        self.apphook_clear()

    def tearDown(self):
        clear_app_resolvers()
        clear_url_caches()

        if self.APP_MODULE in sys.modules:
            del sys.modules[self.APP_MODULE]

        self.reload_urls()
        self.apphook_clear()

    def reload_urls(self):
        from django.conf import settings

        url_modules = [
            "cms.urls",
            self.APP_MODULE,
            settings.ROOT_URLCONF,
        ]

        clear_app_resolvers()
        clear_url_caches()

        for module in url_modules:
            if module in sys.modules:
                del sys.modules[module]

    def _form_view_and_submission_with_apphook_django_gte_111(self, redirect_url):
        if CMS_3_6:
            public_page = self.page.publisher_public
        else:
            public_page = self.page
        try:
            public_placeholder = public_page.placeholders.first()
        except AttributeError:
            public_placeholder = public_page.get_placeholders("en").first()

        public_page_form_plugin = public_placeholder.cmsplugin_set.filter(
            plugin_type="FormPlugin"
        ).first()
        response = self.client.get(self.page.get_absolute_url("en"))

        input_string = '<input type="hidden" name="form_plugin_id" value="{}"'
        self.assertContains(response, input_string.format(public_page_form_plugin.id))  # noqa: E501

        response = self.client.post(
            self.page.get_absolute_url("en"),
            {
                "form_plugin_id": public_page_form_plugin.id,
            },
        )
        self.assertRedirects(
            response, redirect_url, fetch_redirect_response=False
        )  # noqa: E501

    def test_form_view_and_submission_with_apphook_django_gte_111(self):
        self._form_view_and_submission_with_apphook_django_gte_111(self.redirect_url)

    @patch(
        "aldryn_forms.forms.get_random_string",
        lambda length: "aBH7hWEGAihsg9KxctpNRfvEXUoOFpJZigmZETqWWNVs4gENFsL3qva1d4Q93URg",
    )
    @override_settings(ALDRYN_FORMS_MULTIPLE_SUBMISSION_DURATION=30)
    def test_form_view_and_submission_with_apphook_django_gte_111_multiple_steps(self):
        self._form_view_and_submission_with_apphook_django_gte_111(self.redirect_url_with_params)

    def _submit_one_form_instead_multiple(self, redirect_url):
        """Test checks if only one form is send instead of multiple on page together"""
        page = create_page(
            "multiple forms",
            "test_page.html",
            "en",
            published=True,
            apphook="FormsApp",
        )
        placeholder = page.placeholders.get(slot="content")

        form_plugin = add_plugin(
            placeholder,
            "FormPlugin",
            "en",
        )  # noqa: E501

        add_plugin(
            placeholder,
            "EmailField",
            "en",
            name="email_1",
            required=True,
            target=form_plugin,
            label="Submit",
        )

        add_plugin(
            placeholder,
            "SubmitButton",
            "en",
            target=form_plugin,
            label="Submit",
        )

        form_plugin.action_backend = "default"
        form_plugin.save()

        plugin_data2 = {
            "redirect_type": "redirect_to_url",
            "url": redirect_url,
        }

        form_plugin2 = add_plugin(placeholder, "FormPlugin", "en", **plugin_data2)  # noqa: E501

        add_plugin(
            placeholder,
            "SubmitButton",
            "en",
            target=form_plugin2,
            label="Submit",
        )

        form_plugin2.action_backend = "default"
        form_plugin2.save()

        page.publish("en")
        self.reload_urls()
        self.apphook_clear()

        response = self.client.post(
            page.get_absolute_url("en"),
            {
                "form_plugin_id": form_plugin2.id,
                "email_1": "test@test",
            },
        )
        self.assertRedirects(
            response, plugin_data2["url"], fetch_redirect_response=False
        )  # noqa: E501

    @patch(
        "aldryn_forms.forms.get_random_string",
        lambda length: "aBH7hWEGAihsg9KxctpNRfvEXUoOFpJZigmZETqWWNVs4gENFsL3qva1d4Q93URg",
    )
    @override_settings(ALDRYN_FORMS_MULTIPLE_SUBMISSION_DURATION=30)
    def test_view_submit_one_form_instead_multiple_multiple_steps(self):
        self._submit_one_form_instead_multiple(self.redirect_url_with_params)

    def test_view_submit_one_form_instead_multiple(self):
        self._submit_one_form_instead_multiple(self.redirect_url)

    def test_view_submit_one_valid_form_instead_multiple(self):
        """Test checks if only one form is validated instead multiple on a page"""
        page = create_page(
            "multiple forms",
            "test_page.html",
            "en",
            published=True,
            apphook="FormsApp",
        )
        placeholder = page.placeholders.get(slot="content")

        form_plugin = add_plugin(
            placeholder,
            "FormPlugin",
            "en",
        )  # noqa: E501

        add_plugin(
            placeholder,
            "EmailField",
            "en",
            name="email_1",
            required=True,
            target=form_plugin,
        )

        add_plugin(
            placeholder,
            "SubmitButton",
            "en",
            target=form_plugin,
            label="Submit",
        )

        form_plugin.action_backend = "default"
        form_plugin.save()

        form_plugin2 = add_plugin(
            placeholder,
            "FormPlugin",
            "en",
        )  # noqa: E501

        add_plugin(
            placeholder,
            "EmailField",
            "en",
            name="email_2",
            required=True,
            target=form_plugin2,
        )

        add_plugin(
            placeholder,
            "SubmitButton",
            "en",
            target=form_plugin2,
            label="Submit",
        )

        form_plugin2.action_backend = "default"
        form_plugin2.save()

        page.publish("en")
        self.reload_urls()
        self.apphook_clear()

        response = self.client.post(
            page.get_absolute_url("en"),
            {
                "form_plugin_id": form_plugin2.id,
                "email_2": "test@test",
            },
        )

        email_field = '<input type="email" name="{name}"'
        self.assertContains(response, email_field.format(name="email_1"))
        self.assertContains(response, email_field.format(name="email_2"))



from __future__ import unicode_literals

from django.conf import settings
from django.db import migrations, models

import djangocms_text_ckeditor.fields


class Migration(migrations.Migration):

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
        ('aldryn_forms', '0001_initial'),
    ]

    operations = [
        migrations.CreateModel(
            name='EmailNotification',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('theme', models.CharField(help_text='Provides the base theme for the email.', max_length=200, verbose_name='theme', choices=[('default', 'default')])),
                ('to_name', models.CharField(max_length=200, verbose_name='to name', blank=True)),
                ('to_email', models.CharField(max_length=200, verbose_name='to email', blank=True)),
                ('from_name', models.CharField(max_length=200, verbose_name='from name', blank=True)),
                ('from_email', models.CharField(max_length=200, verbose_name='from email', blank=True)),
                ('subject', models.CharField(max_length=200, verbose_name='subject', blank=True)),
                ('body_text', models.TextField(help_text='used when rendering the email in text only mode.', verbose_name='email body (txt)', blank=True)),
                ('body_html', djangocms_text_ckeditor.fields.HTMLField(help_text='used when rendering the email in html.', verbose_name='email body (html)', blank=True)),
                ('to_user', models.ForeignKey(verbose_name='to user', blank=True, to=settings.AUTH_USER_MODEL, null=True, on_delete=models.CASCADE)),
            ],
            options={
            },
            bases=(models.Model,),
        ),
        migrations.CreateModel(
            name='EmailNotificationFormPlugin',
            fields=[
            ],
            options={
                'proxy': True,
            },
            bases=('aldryn_forms.formplugin',),
        ),
        migrations.AddField(
            model_name='emailnotification',
            name='form',
            field=models.ForeignKey(related_name='email_notifications', to='email_notifications.EmailNotificationFormPlugin', on_delete=models.CASCADE),
            preserve_default=True,
        ),
    ]



# import markdown
from typing import Callable, Dict, Optional, Tuple, Union

from django.http import HttpRequest, HttpResponseRedirect, JsonResponse
from django.utils.deprecation import MiddlewareMixin

from aldryn_forms.constants import ALDRYN_FORMS_POST_IDENT_NAME
from aldryn_forms.models import FormPlugin
from aldryn_forms.utils import get_plugin_tree


class HandleHttpPost(MiddlewareMixin):
    """Handle HTTP POST."""

    def process_view(
        self, request: HttpRequest, callback: Callable, callback_args: Tuple[str, ...], callback_kwargs: Dict[str, str]
    ) -> Optional[Union[HttpResponseRedirect, JsonResponse]]:
        """Process view when request method is POST and when the form plugin is found."""

        if request.method != 'POST':
            return None

        # The following code is written according to the function submit_form_view in views.py.
        form_plugin_id = request.POST.get('form_plugin_id')
        if form_plugin_id is None:
            return None
        if not form_plugin_id.isdigit():
            return None

        try:
            form_plugin = get_plugin_tree(FormPlugin, pk=form_plugin_id)
        except FormPlugin.DoesNotExist:
            return None

        form_plugin_instance = form_plugin.get_plugin_instance()[1]
        form = form_plugin_instance.process_form(form_plugin, request)

        if form.is_valid():
            post_ident = form.cleaned_data.get(ALDRYN_FORMS_POST_IDENT_NAME)
            if request.META.get('HTTP_X_REQUESTED_WITH') == "XMLHttpRequest":
                return JsonResponse({
                    "status": "SUCCESS",
                    "post_ident": post_ident,
                    "message": getattr(request, "aldryn_forms_success_message", "OK")
                })
            success_url = form_plugin_instance.get_success_url(instance=form_plugin, post_ident=post_ident)
            if success_url:
                return HttpResponseRedirect(success_url)
        else:
            if request.META.get('HTTP_X_REQUESTED_WITH') == "XMLHttpRequest":
                return JsonResponse({
                    "status": "ERROR",
                    "form": form.errors,
                })

        return None



from django.core.exceptions import ImproperlyConfigured
from django.test import override_settings

from cms.test_utils.testcases import CMSTestCase

from aldryn_forms.action_backends import DefaultAction, EmailAction, NoAction
from aldryn_forms.action_backends_base import BaseAction
from aldryn_forms.utils import action_backend_choices, get_action_backends


class FakeValidBackend(BaseAction):
    verbose_name = 'Fake Valid Backend'

    def form_valid(self, cmsplugin, instance, request, form):
        pass


class FakeValidBackend2(BaseAction):
    verbose_name = 'Another Fake Valid Backend'

    def form_valid(self, cmsplugin, instance, request, form):
        pass


class FakeInvalidBackendNoInheritance():
    verbose_name = 'Fake Invalid Backend (no inheritance)'

    def form_valid(self, cmsplugin, instance, request, form):
        pass


class FakeInvalidBackendNoVerboseName(BaseAction):
    def form_valid(self, cmsplugin, instance, request, form):
        pass


class FakeInvalidBackendNoFormValid(BaseAction):
    verbose_name = 'Fake Invalid Backend (no form_valid() definition)'


class GetActionsTestCase(CMSTestCase):
    def test_default_backends(self):
        expected = {
            'default': DefaultAction,
            'email_only': EmailAction,
            'none': NoAction,
        }

        backends = get_action_backends()

        self.assertDictEqual(backends, expected)

    @override_settings(ALDRYN_FORMS_ACTION_BACKENDS={
        'default': 'tests.test_utils.FakeValidBackend',
        'x': 'tests.test_utils.FakeValidBackend2',
    })
    def test_override_valid(self):
        expected = {
            'default': FakeValidBackend,
            'x': FakeValidBackend2,
        }

        backends = get_action_backends()

        self.assertDictEqual(backends, expected)

    @override_settings(ALDRYN_FORMS_ACTION_BACKENDS={
        'default': 'tests.test_utils.FakeValidBackend',
        'x' * 100: 'tests.test_utils.FakeValidBackend2',
    })
    def test_override_invalid_keys_too_big(self):
        self.assertRaises(ImproperlyConfigured, get_action_backends)

    @override_settings(ALDRYN_FORMS_ACTION_BACKENDS={
        'default': 'tests.whatever.something.terribly.Wrong',
    })
    def test_override_invalid_path_to_class_not_found(self):
        self.assertRaises(ImproperlyConfigured, get_action_backends)

    @override_settings(ALDRYN_FORMS_ACTION_BACKENDS={
        'default': 'tests.test_utils.FakeInvalidBackendNoInheritance',
    })
    def test_override_invalid_class_does_not_inherit_from_base_action_backend(self):
        self.assertRaises(ImproperlyConfigured, get_action_backends)

    @override_settings(ALDRYN_FORMS_ACTION_BACKENDS={
        'custom': 'tests.test_utils.FakeValidBackend',
    })
    def test_override_invalid_key_default_missing(self):
        self.assertRaises(ImproperlyConfigured, get_action_backends)

    @override_settings(ALDRYN_FORMS_ACTION_BACKENDS={
        'default': 'tests.test_utils.FakeInvalidBackendNoVerboseName',
    })
    def test_override_invalid_class_does_not_define_verbose_name(self):
        self.assertRaises(ImproperlyConfigured, get_action_backends)

    @override_settings(ALDRYN_FORMS_ACTION_BACKENDS={
        'default': 'tests.test_utils.FakeInvalidBackendNoFormValid',
    })
    def test_override_invalid_class_does_not_define_form_valid(self):
        self.assertRaises(ImproperlyConfigured, get_action_backends)


class ActionChoicesTestCase(CMSTestCase):
    def test_default_backends(self):
        expected = [
            ('none', 'No action'),
            ('email_only', 'Only send email'),
            ('default', 'Save to site administration and send email'),
        ]

        choices = action_backend_choices()

        self.assertEqual(choices, expected)

    @override_settings(ALDRYN_FORMS_ACTION_BACKENDS={
        'default': 'tests.test_utils.FakeValidBackend',
        'x': 'tests.test_utils.FakeValidBackend2',
    })
    def test_override_valid(self):
        expected = [
            ('x', 'Another Fake Valid Backend'),
            ('default', 'Fake Valid Backend'),
        ]

        choices = action_backend_choices()

        self.assertEqual(choices, expected)



# Generated by Django 1.9.8 on 2016-08-21 14:39
from __future__ import unicode_literals

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('aldryn_forms', '0005_auto_20160821_1026'),
    ]

    operations = [
        migrations.AlterField(
            model_name='emailfieldplugin',
            name='cmsplugin_ptr',
            field=models.OneToOneField(on_delete=django.db.models.deletion.CASCADE, parent_link=True, primary_key=True, related_name='aldryn_forms_emailfieldplugin', serialize=False, to='cms.CMSPlugin'),
        ),
        migrations.AlterField(
            model_name='fieldplugin',
            name='cmsplugin_ptr',
            field=models.OneToOneField(on_delete=django.db.models.deletion.CASCADE, parent_link=True, primary_key=True, related_name='aldryn_forms_fieldplugin', serialize=False, to='cms.CMSPlugin'),
        ),
        migrations.AlterField(
            model_name='fieldsetplugin',
            name='cmsplugin_ptr',
            field=models.OneToOneField(on_delete=django.db.models.deletion.CASCADE, parent_link=True, primary_key=True, related_name='aldryn_forms_fieldsetplugin', serialize=False, to='cms.CMSPlugin'),
        ),
        migrations.AlterField(
            model_name='fileuploadfieldplugin',
            name='cmsplugin_ptr',
            field=models.OneToOneField(on_delete=django.db.models.deletion.CASCADE, parent_link=True, primary_key=True, related_name='aldryn_forms_fileuploadfieldplugin', serialize=False, to='cms.CMSPlugin'),
        ),
        migrations.AlterField(
            model_name='formbuttonplugin',
            name='cmsplugin_ptr',
            field=models.OneToOneField(on_delete=django.db.models.deletion.CASCADE, parent_link=True, primary_key=True, related_name='aldryn_forms_formbuttonplugin', serialize=False, to='cms.CMSPlugin'),
        ),
        migrations.AlterField(
            model_name='formplugin',
            name='cmsplugin_ptr',
            field=models.OneToOneField(on_delete=django.db.models.deletion.CASCADE, parent_link=True, primary_key=True, related_name='aldryn_forms_formplugin', serialize=False, to='cms.CMSPlugin'),
        ),
        migrations.AlterField(
            model_name='imageuploadfieldplugin',
            name='cmsplugin_ptr',
            field=models.OneToOneField(on_delete=django.db.models.deletion.CASCADE, parent_link=True, primary_key=True, related_name='aldryn_forms_imageuploadfieldplugin', serialize=False, to='cms.CMSPlugin'),
        ),
        migrations.AlterField(
            model_name='textareafieldplugin',
            name='cmsplugin_ptr',
            field=models.OneToOneField(on_delete=django.db.models.deletion.CASCADE, parent_link=True, primary_key=True, related_name='aldryn_forms_textareafieldplugin', serialize=False, to='cms.CMSPlugin'),
        ),
    ]



# Generated by Django 4.2.14 on 2024-08-01 13:21

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("aldryn_forms", "0017_alter_datefieldplugin_cmsplugin_ptr_and_more")
    ]

    operations = [
        migrations.AddField(
            model_name="formplugin",
            name="use_form_action",
            field=models.BooleanField(
                default=False,
                help_text='Use the URL or CMS Page in the form attribute "action".',
                verbose_name="Form action attribute",
            ),
        ),
    ]



from django.core import exceptions
from django.db import models
from django.utils.translation import gettext_lazy as _

from .utils import parse_size
from .widgets import FileSizeWidget


class FileSizeField(models.BigIntegerField):
    default_error_messages = {
        'invalid': _(u'Incorrect file size format.'),
    }

    def formfield(self, **kwargs):
        kwargs['widget'] = FileSizeWidget
        kwargs['error_messages'] = self.default_error_messages
        return super(FileSizeField, self).formfield(**kwargs)

    def to_python(self, value):
        if value is None:
            return None
        try:
            return parse_size(value)
        except ValueError:
            raise exceptions.ValidationError(self.error_messages['invalid'])



from django import template
from django.utils import encoding
from django.utils.safestring import mark_safe

import markdown as markdown_module


register = template.Library()


@register.simple_tag(takes_context=True)
def render_notification_text(context, email_notification, email_type):
    text_context = context.get('text_context')

    if not text_context or not email_notification:
        return

    render_func = 'render_%s' % email_type
    message = getattr(email_notification, render_func)(context=text_context)
    return mark_safe(message)


@register.simple_tag()
def render_form_widget(field, **kwargs):
    if "class" in kwargs and field.errors:
        if kwargs["class"]:
            kwargs["class"] += " "
        kwargs["class"] += "has-error"
    markup = field.as_widget(attrs=kwargs)
    return mark_safe(markup)


@register.filter()
def force_text(val):
    return encoding.force_str(val)


@register.filter()
def force_text_list(val):
    return [encoding.force_str(v) for v in val]


@register.filter()
def markdown(text):
    return mark_safe(markdown_module.markdown(text))



from django.utils.translation import gettext

from .helpers import get_email_template_name


class BaseNotificationConf(object):
    # list of extra context keys available for email content/headers
    # format should be:
    # [(Title, [(field_1, field_1_label), (field_2, field_2_label)])]
    # fields have to be valid python identifiers
    custom_context_choices = None

    # should we allow the user to configure the email txt format?
    # this is not the same as the html option above
    # we can't disable txt format but we can choose to not display
    # it on the admin.
    txt_email_format_configurable = True
    # default lookup template name, doesn't includes the extension
    txt_email_template_name = 'body'
    # should we send out an html version of the email?
    # by design if the html version of email is enabled
    # then is configurable via the admin.
    html_email_format_enabled = True
    # default lookup template name, doesn't includes the extension
    html_email_template_name = 'body'

    def __init__(self, form_plugin):
        self.form_plugin = form_plugin

    def get_txt_email_template_name(self):
        return get_email_template_name(
            name=self.txt_email_template_name, suffix='txt')

    def get_html_email_template_name(self):
        return get_email_template_name(
            name=self.html_email_template_name, suffix='html')

    def get_context(self, form):
        text_context = form.get_cleaned_data(is_confirmation=True)
        text_context['form_name'] = self.form_plugin.name

        if self.custom_context_choices:
            custom_context = self.get_custom_context(form)
            text_context.update(custom_context)
        return text_context

    def get_custom_context(self, form):
        return {}

    def get_context_keys_as_choices(self):
        choices = [
            (
                gettext('Fields'),
                list(self.form_plugin.get_form_fields_as_choices())
            ),
        ]

        if self.custom_context_choices:
            choices += list(self.custom_context_choices)
        return choices


class DefaultNotificationConf(BaseNotificationConf):
    html_email_format_enabled = True
    txt_email_format_configurable = True



from django.core.management.base import BaseCommand

from aldryn_forms.models import SubmittedToBeSent
from aldryn_forms.utils import send_postponed_notifications


class Command(BaseCommand):
    help = "Send postponed emails."

    def handle(self, *args, **options):
        for instance in SubmittedToBeSent.objects.all():
            if send_postponed_notifications(instance):
                instance.delete()



from cms.cms_plugins import AliasPlugin


def get_user_name(user):
    try:
        name = user.get_full_name()
    except AttributeError:
        # custom user model
        name = ''
    return name


def is_form_element(plugin):
    # import here due because of circular imports
    from .cms_plugins import FormElement

    # cms_plugins.CMSPlugin subclass
    cms_plugin = plugin.get_plugin_class_instance(None)
    is_orphan_plugin = cms_plugin.model != plugin.__class__
    plugin_class = plugin.get_plugin_class()
    if plugin.get_plugin_class() == AliasPlugin:
        plugin_class = plugin.plugin.get_plugin_class()
    is_element_subclass = issubclass(plugin_class, FormElement)
    return (not is_orphan_plugin) and is_element_subclass



# Generated by Django 2.0 on 2019-01-04 11:42

from django.conf import settings
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('aldryn_forms', '0011_auto_20180110_1300'),
    ]

    operations = [
        migrations.AlterField(
            model_name='formplugin',
            name='recipients',
            field=models.ManyToManyField(blank=True, help_text='People who will get the form content via e-mail.', limit_choices_to={'is_staff': True}, to=settings.AUTH_USER_MODEL, verbose_name='Recipients'),
        ),
    ]



from email.utils import formataddr

from django.conf import settings
from django.contrib import admin
from django.template.loader import render_to_string
from django.urls import reverse
from django.utils.encoding import force_str
from django.utils.module_loading import import_string
from django.utils.translation import gettext_lazy as _

from import_export.fields import Field
from import_export.resources import Resource


str_dunder_method = '__str__'


class FieldKey:
    """Field key is pair of names - parent, child."""

    def __init__(self, parent, child):
        self.parent = parent
        self.child = child

    def __str__(self):
        return "{}+{}".format(self.parent, self.child)


class AldrynFormExportField(Field):
    """AldrynForm export field."""

    def get_value(self, obj):
        if isinstance(self.attribute, FieldKey):
            return obj.get(self.attribute.parent, {}).get(self.attribute.child)
        return obj.get(self.attribute)


class BaseFormSubmissionAdmin(admin.ModelAdmin):
    date_hierarchy = 'sent_at'
    list_display = [str_dunder_method, 'sent_at', 'language', 'display_data']
    list_filter = ['name', 'language']
    readonly_fields = [
        'name',
        'get_data_for_display',
        'language',
        'sent_at',
        'get_recipients_for_display',
    ]

    # (Field name, Field label, json data)
    export_fields = (
        ('name', _('form name'), False),
        ('language', _('form language'), False),
        ('sent_at', _('sent at'), False),
        ('data', _('form data'), True),
        ('recipients', _('users notified'), True),
    )

    def has_add_permission(self, request):
        return False

    def get_data_for_display(self, obj):
        data = obj.get_form_data()
        html = render_to_string(
            'admin/aldryn_forms/display/submission_data.html',
            {'data': data}
        )
        return html
    get_data_for_display.allow_tags = True
    get_data_for_display.short_description = _('data')

    def get_recipients(self, obj):
        recipients = obj.get_recipients()
        formatted = [formataddr((recipient.name, recipient.email))
                     for recipient in recipients]
        return formatted

    def get_recipients_for_display(self, obj):
        people_list = self.get_recipients(obj)
        html = render_to_string(
            'admin/aldryn_forms/display/recipients.html',
            {'people': people_list},
        )
        return html
    get_recipients_for_display.allow_tags = True
    get_recipients_for_display.short_description = _('people notified')

    def get_urls(self):
        from django.urls import path

        url_patterns = [
            path('export/', self.admin_site.admin_view(self.get_form_export_view()), name=self.get_admin_url('export')),
        ]
        return url_patterns + super(BaseFormSubmissionAdmin, self).get_urls()

    def get_admin_url(self, name):
        try:
            model_name = self.model._meta.model_name
        except AttributeError:
            # django <= 1.5 compat
            model_name = self.model._meta.module_name

        url_name = "%s_%s_%s" % (self.model._meta.app_label, model_name, name)
        return url_name

    def get_admin_context(self, form=None, title=None):
        opts = self.model._meta

        context = {
            'media': self.media,
            'has_change_permission': True,
            'opts': opts,
            'root_path': reverse('admin:index'),
            'current_app': self.admin_site.name,
            'app_label': opts.app_label,
        }

        if form:
            context['adminform'] = form
            context['media'] += form.media

        if title:
            context['original'] = title
        return context

    def get_form_export_view(self):
        raise NotImplementedError

    def export_field_parse_data(self, submission):
        """Parse export form field data."""
        fields, values = {}, {}
        for serialized_form_field in submission.get_form_data():
            if serialized_form_field.value:
                fields[serialized_form_field.field_id] = serialized_form_field.label
                values[serialized_form_field.field_id] = serialized_form_field.value
        return fields, values

    def export_field_parse_recipients(self, submission):
        """Parse export form field recipients."""
        fields, values = {'email': _("E-mail")}, {}
        for recipient in submission.get_recipients():
            values['email'] = recipient.email
            if recipient.name:
                values['name'] = recipient.name
                if 'name' not in fields:
                    fields['name'] = _('Name')
        return fields, values

    def export_dataset_and_labels(self, queryset):
        """Collect fields from JSON data."""
        dataset = []
        extra_field_labels = {}
        cols = {name: label for name, label, json_data in self.export_fields}
        unique_codes = []
        for submission in queryset:
            data_item = {}
            for field_name, field_label, field_json_data in self.export_fields:
                field_value = getattr(submission, field_name)
                if field_value is None or field_value == "":
                    continue
                if field_json_data:
                    fnc = getattr(self, "export_field_parse_{}".format(field_name), None)
                    if fnc is not None:
                        field_columns, field_values = fnc(submission)
                        data_item[field_name] = field_values
                        for name, label in field_columns.items():
                            field_key = FieldKey(field_name, name)
                            code = str(field_key)
                            if code not in unique_codes:
                                unique_codes.append(code)
                                if field_name not in extra_field_labels:
                                    extra_field_labels[field_name] = {}
                                extra_field_labels[field_name][field_key] = "{} / {}".format(cols[field_name], label)
                    else:
                        data_item[field_name] = field_value
                else:
                    data_item[field_name] = field_value
            dataset.append(data_item)
        return dataset, extra_field_labels

    def export_data(self, export_type, queryset):
        """Export data into format defined by export_type."""
        extra_field_labels = {}
        dataset, extra_field_labels = self.export_dataset_and_labels(queryset)

        fields = []
        headers = []
        for name, label, json_data in self.export_fields:
            if json_data:
                for code, label in extra_field_labels.get(name, {}).items():
                    fields.append(AldrynFormExportField(attribute=code))
                    headers.append(force_str(label))
            else:
                fields.append(AldrynFormExportField(attribute=name))
                headers.append(force_str(label))

        resource = Resource()
        resource.get_export_headers = lambda: headers
        for field in fields:
            resource.fields[force_str(field.attribute)] = field

        return getattr(resource.export(dataset), export_type)

    def display_data(self, obj):
        if hasattr(settings, 'ALDRYN_FORMS_SUBMISSION_LIST_DISPLAY_FIELD'):
            submission_field = import_string(settings.ALDRYN_FORMS_SUBMISSION_LIST_DISPLAY_FIELD)
            return submission_field(obj)
        return ''
    display_data.short_description = _('data')



import logging

from django.conf import settings
from django.utils.translation import gettext_lazy as _

from .action_backends_base import BaseAction
from .constants import ALDRYN_FORMS_MULTIPLE_SUBMISSION_DURATION


logger = logging.getLogger(__name__)


class DefaultAction(BaseAction):
    verbose_name = _('Save to site administration and send email')

    def form_valid(self, cmsplugin, instance, request, form):
        duration = getattr(settings, ALDRYN_FORMS_MULTIPLE_SUBMISSION_DURATION, 0)
        if duration:
            recipients = cmsplugin.postpone_send_notifications(instance, form)
        else:
            recipients = cmsplugin.send_notifications(instance, form)
        form.instance.set_recipients(recipients)
        form.save()
        cmsplugin.send_success_message(instance, request)


class EmailAction(BaseAction):
    verbose_name = _('Only send email')

    def form_valid(self, cmsplugin, instance, request, form):
        duration = getattr(settings, ALDRYN_FORMS_MULTIPLE_SUBMISSION_DURATION, 0)
        if duration:
            recipients = cmsplugin.postpone_send_notifications(instance, form)
            logger.info(f'Pospone sent email notifications to {len(recipients)} recipients.')
        else:
            recipients = cmsplugin.send_notifications(instance, form)
            logger.info(f'Sent email notifications to {len(recipients)} recipients.')
        cmsplugin.send_success_message(instance, request)


class NoAction(BaseAction):
    verbose_name = _('No action')

    def form_valid(self, cmsplugin, instance, request, form):
        form_id = form.form_plugin.id
        logger.info(f'Not persisting data for "{form_id}" since action_backend is set to "none"')



from django.conf import settings
from django.core.management.base import BaseCommand
from django.utils.timezone import now as django_timezone_now
from django.utils.timezone import timedelta

from aldryn_forms.constants import ALDRYN_FORMS_MULTIPLE_SUBMISSION_DURATION
from aldryn_forms.models import FormSubmission


class Command(BaseCommand):
    help = "Remove expired post idents."

    def handle(self, *args, **options):
        duration = getattr(settings, ALDRYN_FORMS_MULTIPLE_SUBMISSION_DURATION, 0)
        if duration:
            expire = django_timezone_now() - timedelta(minutes=duration)
            FormSubmission.objects.filter(
                post_ident__isnull=False, sent_at__gt=expire
            ).update(post_ident=None)



# Generated by Django 4.2.6 on 2025-02-12 12:58

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('aldryn_forms', '0018_formplugin_use_form_action'),
    ]

    operations = [
        migrations.CreateModel(
            name='SubmittedToBeSent',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(db_index=True, editable=False, max_length=255, verbose_name='form name')),
                ('data', models.TextField(blank=True, editable=False)),
                ('recipients', models.TextField(blank=True, editable=False, help_text='People who got a notification when form was submitted.', verbose_name='users notified')),
                ('language', models.CharField(choices=[('cs', 'cs'), ('en', 'en')], default='cs', max_length=10, verbose_name='form language')),
                ('form_url', models.CharField(blank=True, max_length=255, verbose_name='form url')),
                ('sent_at', models.DateTimeField(auto_now_add=True)),
                ('post_ident', models.CharField(blank=True, max_length=64, null=True)),
            ],
            options={
                'verbose_name': 'Submitted form to be sent',
                'verbose_name_plural': 'Submitted forms to be sent',
                'ordering': ['-sent_at'],
            },
        ),
        migrations.AddField(
            model_name='formsubmission',
            name='post_ident',
            field=models.CharField(blank=True, max_length=64, null=True),
        ),
    ]



from django.core.exceptions import ValidationError
from django.db import IntegrityError
from django.test import TestCase

from cms.api import add_plugin
from cms.models import Placeholder

from filer.models import Folder

from aldryn_forms.models import (
    FileUploadFieldPlugin, ImageUploadFieldPlugin,
    MultipleFilesUploadFieldPlugin, Option,
)


class OptionTestCase(TestCase):
    def setUp(self):
        super(TestCase, self).setUp()
        self.placeholder = Placeholder.objects.create(slot='test')
        self.field = add_plugin(self.placeholder, 'SelectField', 'en')

    def test_position_organic_ordering(self):
        ''' Tests that no manual ordering leads to an organic ordering: first added, first displayed. '''
        self.field.option_set.create(value='one')
        self.field.option_set.create(value='two')
        self.field.option_set.create(value='three')
        self.field.option_set.create(value='four')
        self.field.option_set.create(value='five')

        option1, option2, option3, option4, option5 = self.field.option_set.all()

        self.assertEqual(option1.value, 'one')
        self.assertEqual(option1.position, 10)
        self.assertEqual(option2.value, 'two')
        self.assertEqual(option2.position, 20)
        self.assertEqual(option3.value, 'three')
        self.assertEqual(option3.position, 30)
        self.assertEqual(option4.value, 'four')
        self.assertEqual(option4.position, 40)
        self.assertEqual(option5.value, 'five')
        self.assertEqual(option5.position, 50)

    def test_position_manual_ordering(self):
        self.field.option_set.create(position=100, value='below $10')
        self.field.option_set.create(position=200, value='between $10 and $50')
        self.field.option_set.create(position=10, value='super promo: below $1!')
        self.field.option_set.create(position=300, value='$50+ because you are rich')
        self.field.option_set.create(position=1, value='items for free (but they are broken - sorry)')

        option1, option2, option3, option4, option5 = self.field.option_set.all()

        self.assertEqual(option1.value, 'items for free (but they are broken - sorry)')
        self.assertEqual(option1.position, 1)
        self.assertEqual(option2.value, 'super promo: below $1!')
        self.assertEqual(option2.position, 10)
        self.assertEqual(option3.value, 'below $10')
        self.assertEqual(option3.position, 100)
        self.assertEqual(option4.value, 'between $10 and $50')
        self.assertEqual(option4.position, 200)
        self.assertEqual(option5.value, '$50+ because you are rich')
        self.assertEqual(option5.position, 300)

    def test_hybrid_ordering(self):
        self.field.option_set.create(position=31415, value='But after a while I got lazy')
        self.field.option_set.create(value='and so I didnt wanna')
        self.field.option_set.create(value='set this ordering')
        self.field.option_set.create(value='anymore')
        self.field.option_set.create(position=42, value='I started this ordering manually')

        option1, option2, option3, option4, option5 = self.field.option_set.all()

        self.assertEqual(option1.value, 'I started this ordering manually')
        self.assertEqual(option1.position, 42)
        self.assertEqual(option2.value, 'But after a while I got lazy')
        self.assertEqual(option2.position, 31415)
        self.assertEqual(option3.value, 'and so I didnt wanna')
        self.assertEqual(option3.position, 31425)
        self.assertEqual(option4.value, 'set this ordering')
        self.assertEqual(option4.position, 31435)
        self.assertEqual(option5.value, 'anymore')
        self.assertEqual(option5.position, 31445)

    def test_position_is_not_nullable(self):
        self.field.option_set.create(position=950, value='950')

        # Noise
        another_field = add_plugin(self.placeholder, 'SelectField', 'en')
        another_field.option_set.create(position=1000, value='1000 for another field so it does not matter')

        option1 = self.field.option_set.create(position=1, value='test')
        option1.position = None
        option1.save()
        self.assertEqual(option1.position, 960)  # We force a value for it on Option.save

        self.assertRaises(IntegrityError, Option.objects.update, position=None)  # See? Not nullable


class FileUploadFieldPluginTest(TestCase):

    def test(self):
        field = FileUploadFieldPlugin(max_size=42)
        self.assertEqual(field.max_size, 42)
        self.assertIsNone(field.enable_js)
        field = FileUploadFieldPlugin(enable_js=True)
        self.assertTrue(field.enable_js)
        self.assertIsNone(field.max_size)
        self.assertIsNone(field.accepted_types)

    def test_accepted_types(self):
        folder = Folder.objects.create(name='Test')
        field = FileUploadFieldPlugin.objects.create(upload_to=folder, accepted_types='.pdf text/plain image/*')
        self.assertIsNone(field.full_clean())

    def test_accepted_types_fails(self):
        folder = Folder.objects.create(name='Test')
        field = FileUploadFieldPlugin.objects.create(upload_to=folder, accepted_types='pdf text')
        with self.assertRaisesMessage(ValidationError, "{'accepted_types': ['pdf text is not mimetype.']}"):
            field.full_clean()


class MultipleFilesUploadFieldPluginTest(TestCase):

    def test(self):
        field = MultipleFilesUploadFieldPlugin(max_size=42)
        self.assertEqual(field.max_size, 42)
        self.assertIsNone(field.enable_js)
        field = MultipleFilesUploadFieldPlugin(enable_js=True)
        self.assertTrue(field.enable_js)
        self.assertIsNone(field.max_size)
        self.assertIsNone(field.accepted_types)

    def test_accepted_types(self):
        folder = Folder.objects.create(name='Test')
        field = MultipleFilesUploadFieldPlugin.objects.create(
            upload_to=folder, accepted_types='.pdf text/plain image/*')
        self.assertIsNone(field.full_clean())

    def test_accepted_types_fails(self):
        folder = Folder.objects.create(name='Test')
        field = MultipleFilesUploadFieldPlugin.objects.create(upload_to=folder, accepted_types='pdf text')
        with self.assertRaisesMessage(ValidationError, "{'accepted_types': ['pdf text is not mimetype.']}"):
            field.full_clean()


class ImageUploadFieldPluginTest(TestCase):

    def test(self):
        field = ImageUploadFieldPlugin(max_size=42)
        self.assertEqual(field.max_size, 42)
        self.assertIsNone(field.enable_js)
        field = ImageUploadFieldPlugin(enable_js=True)
        self.assertTrue(field.enable_js)
        self.assertIsNone(field.max_size)
